@@ -23,6 +23,7 @@ import abc
 import logging
 import os
 import sys
+from collections import Counter
 from typing import Iterable, Generic, Optional, T, Type
 
 import click
@@ -33,6 +34,7 @@ from nr.utils.git import Git
 from termcolor import colored
 
 from shut.changelog.manager import ChangelogManager
+from shut.changelog.v3 import Changelog
 from shut.commands import project
 from shut.model import AbstractProjectModel, Project
 from shut.model.version import bump_version, parse_version, Version
@@ -67,20 +69,23 @@ class VersionBumpData(Generic[T], metaclass=abc.ABCMeta):
     self.project = project
     self.obj = obj
 
-  def post_init(self) -> None:
+  def loaded(self) -> None:
     pass
 
   @abc.abstractmethod
   def get_snapshot_version(self) -> Version:
     pass
 
+  def get_version_refs(self) -> Iterable[VersionRef]:
+    yield from get_version_refs(self.obj)
+
   def bump_to_version(self, target_version: Version) -> Iterable[str]:
     """
     Called to bump to the specified *target_version*. The default implementation uses the
     version refs provided by #get_version_refs() to bump.
     """
 
-    version_refs = list(get_version_refs(self.obj))
+    version_refs = list(self.get_version_refs())
 
     print()
     print(f'bumping {len(version_refs)} version reference(s)')
@@ -108,17 +113,19 @@ class VersionBumpData(Generic[T], metaclass=abc.ABCMeta):
     changed_files = set(x.filename for x in version_refs)
 
     # Release the staged changelog.
-    manager = ChangelogManager(self.obj.get_changelog_directory())
-    if manager.unreleased.exists():
-      changed_files.add(manager.unreleased.filename)
-      if self.args.dry:
-        changelog = manager.version(target_version)
-      else:
-        changelog = manager.release(target_version)
-      changed_files.add(changelog.filename)
+    managers = list(self.get_changelog_managers())
+    managers = [m for m in managers if m.unreleased.exists()]
+    if managers:
       print()
-      print('release staged changelog')
-      print(f'  {colored(os.path.relpath(manager.unreleased.filename), "cyan")} → {os.path.relpath(changelog.filename)}')
+      print('release staged changelog' + ('s' if len(managers) > 1 else ''))
+      for manager in managers:
+        changed_files.add(manager.unreleased.filename)
+        if self.args.dry:
+          changelog = manager.version(target_version)
+        else:
+          changelog = manager.release(target_version)
+        changed_files.add(changelog.filename)
+        print(f'  {colored(os.path.relpath(manager.unreleased.filename), "cyan")} → {os.path.relpath(changelog.filename)}')
 
     return changed_files
 
@@ -129,6 +136,9 @@ class VersionBumpData(Generic[T], metaclass=abc.ABCMeta):
     must be returned.
     """
 
+  def get_changelog_managers(self) -> Iterable[ChangelogManager]:
+    yield ChangelogManager(self.obj.get_changelog_directory())
+
 
 def make_bump_command(
   data_class: Type[VersionBumpData[AbstractProjectModel]],
@@ -153,6 +163,7 @@ def make_bump_command(
   def bump(**kwargs):
     args = Args(**kwargs)
     data = data_class(args, project, project.load_or_exit(expect=model_type))
+    data.loaded()
     do_bump(args, data)
 
   return bump
@@ -168,8 +179,21 @@ def do_bump(args: Args, data: VersionBumpData[AbstractProjectModel]) -> None:
   if len(provided_args) > 1:
     sys.exit('error: conflicting options: {}'.format(provided_args))
   elif not provided_args:
-    # TODO(NiklasRosenstein): Bump based on changelog
-    sys.exit('error: missing version argument or bump option')
+    print()
+    print('figuring bump mode from changelog')
+    entries = []
+    for manager in data.get_changelog_managers():
+      if manager.unreleased.exists():
+        entries += manager.unreleased.entries
+    if not entries:
+      sys.exit('error: no changelog entries found')
+    counter = Counter(entry.type_.name for entry in entries)
+    bump_mode = max(entry.type_.bump_mode for entry in entries)
+    assert hasattr(args, bump_mode.name), bump_mode.name
+    setattr(args, bump_mode.name, True)
+    print('  {} → {}'.format(
+      ', '.join(f'{c} {colored(k, "cyan")}' for k, c in counter.items()),
+      colored(bump_mode.name, "blue", attrs=["bold"])))
 
   version_refs = list(get_version_refs(data.obj))
   if not version_refs:
@@ -222,10 +246,19 @@ def do_bump(args: Args, data: VersionBumpData[AbstractProjectModel]) -> None:
   if args.tag:
     print()
     git = Git()
-    if any(f.mode == 'A' for f in git.porcelain()):
+    files = list(git.porcelain())
+
+    # We require that no files are currently staged.
+    if any(f.mode == 'A' for f in files):
       logger.error('cannot tag with non-empty staging area')
       exit(1)
 
+    # TODO: If this step errors (e.g. for example because the unreleased changelog file
+    #   was not yet tracked by Git, making Git complain that it doesn't know about the
+    #   file and it doesn't exits anymore when we add it down below), it's a bit painful
+    #   to recover from it because it's a manual process of reverting the version number
+    #   bumps.
+
     tag_name = data.obj.get_tag(new_version)
     print(f'tagging {tag_name}')
 
