@@ -23,7 +23,7 @@ import os
 import shlex
 import subprocess as sp
 import sys
-from typing import List
+from typing import List, Optional, Set, Tuple
 
 import click
 from nr.stream import concat
@@ -35,54 +35,16 @@ from . import pkg
 from .. import project
 
 
-class Requirement:
-
-  def __init__(self, name: str, develop: bool = False, extras: str = None) -> None:
-    self.name = name
-    self.develop = develop
-    self.extras = extras
-
-  def __str__(self):
-    return ' '.join(self.to_args())
-
-  def __repr__(self):
-    return 'Requirement({})'.format(str(self))
-
-  def to_args(self) -> List[str]:
-    args = [self.name + ('[{}]'.format(self.extras) if self.extras else '')]
-    if self.develop:
-      args.insert(0, '-e')
-    return args
-
-
-@pkg.command()
-@click.option('--develop/--no-develop', default=True,
-  help='Install in develop mode (default: true)')
-@click.option('--inter-deps/--no-inter-deps', default=True,
-  help='Install package inter dependencies from inside the same monorepo (default: true)')
-@click.option('--extra', help='Specify one or more extras to install.')
-@click.option('-U', '--upgrade', is_flag=True, help='Upgrade all packages (forwarded to pip install).')
-@click.option('-q', '--quiet', is_flag=True, help='Quiet install')
-@click.option('--pip', help='Override the command to run Pip. Defaults to "python -m pip" or the PIP variable.')
-@click.option('--pip-args', help='Additional arguments to pass to Pip.')
-@click.option('--dry', is_flag=True, help='Print the Pip command to stdout instead of running it.')
-def install(develop, inter_deps, extra, upgrade, quiet, pip, pip_args, dry):
-  """
-  Install the package using `python -m pip`. If the package is part of a mono repository,
-  inter-dependencies will be installed from the mono repsitory rather than from PyPI.
-
-  The command used to invoke Pip can be overwritten using the `PIP` environment variable.
-  """
-
-  package = project.load_or_exit(expect=PackageModel)
-  extras = set((extra or '').split(','))
+def collect_requirements(
+  package: bool,
+  inter_deps: bool,
+  extra: Optional[Set[str]],
+) -> RequirementsList:
   reqs = RequirementsList()
 
   # Pip does not understand "test" as an extra and does not have an option to
   # install test requirements.
-  if 'test' in extras:
-    extras.discard('test')
-    extra = ','.join(extras)
+  if extra and 'test' in extra:
     reqs += package.test_requirements
 
   reqs.append(VendoredRequirement(VendoredRequirement.Type.Path, package.get_directory()))
@@ -99,17 +61,63 @@ def install(develop, inter_deps, extra, upgrade, quiet, pip, pip_args, dry):
               '{} does not match the present version {}'
               .format(dep.name, ref.version_selector, dep.version), file=sys.stderr)
       else:
-        reqs.insert(0, Requirement(dep.get_directory(), develop))
+        reqs.insert(0, VendoredRequirement(VendoredRequirement.Type.Path, dep.get_directory()))
+
+  return reqs
+
+
+def run_install(
+  pip: Optional[str],
+  reqs: List[Tuple[str, RequirementsList]],
+  develop: bool,
+  upgrade: bool,
+  quiet: bool,
+  dry: bool,
+  pip_args: Optional[str],
+) -> None:
 
   pip_bin = shlex.split(os.getenv('PIP', pip or 'python -m pip'))
-  command = pip_bin + ['install'] + reqs.to_pip_args(package.get_directory(), develop)
+  command = pip_bin + ['install']
+  for directory, req in reqs:
+    command += req.to_pip_args(directory, develop)
   if upgrade:
     command.append('--upgrade')
   if quiet:
     command.append('--quiet')
   command += shlex.split(pip_args) if pip_args else []
 
+  command_str = ' '.join(map(shlex.quote, command))
   if dry:
-    print(' '.join(map(shlex.quote, command)))
+    print(command_str)
   else:
     sys.exit(sp.call(command))
+
+
+def split_extras(extras: str) -> Set[str]:
+  result = set(map(str.strip, extras.split(',')))
+  result.discard('')
+  return result
+
+
+@pkg.command()
+@click.option('--develop/--no-develop', default=True,
+  help='Install in develop mode (default: true)')
+@click.option('--inter-deps/--no-inter-deps', default=True,
+  help='Install package inter dependencies from inside the same monorepo (default: true)')
+@click.option('--extra', type=split_extras, help='Specify one or more extras to install.')
+@click.option('-U', '--upgrade', is_flag=True, help='Upgrade all packages (forwarded to pip install).')
+@click.option('-q', '--quiet', is_flag=True, help='Quiet install')
+@click.option('--pip', help='Override the command to run Pip. Defaults to "python -m pip" or the PIP variable.')
+@click.option('--pip-args', help='Additional arguments to pass to Pip.')
+@click.option('--dry', is_flag=True, help='Print the Pip command to stdout instead of running it.')
+def install(develop, inter_deps, extra, upgrade, quiet, pip, pip_args, dry):
+  """
+  Install the package using `python -m pip`. If the package is part of a mono repository,
+  inter-dependencies will be installed from the mono repsitory rather than from PyPI.
+
+  The command used to invoke Pip can be overwritten using the `PIP` environment variable.
+  """
+
+  package = project.load_or_exit(expect=PackageModel)
+  reqs = collect_requirements(package, inter_deps, extra)
+  run_install(pip, [(package.get_directory(), reqs)], develop, upgrade, quiet, dry, pip_args)
